

# 初始化代码写这里

# import dknovautils.myadd
# import dknovautils.mysubtract

# from .myadd import m_add
# import mysubtract


from dknovautils.commons import *

from dknovautils.dkat import *

from dknovautils.dkipy import *

from dknovautils.dkfiles import *












from dknovautils.dk_imports import *


from dknovautils.dkat import *

from dknovautils.dkipy import *


def write_async():
    AT.unimplemented()
    pass


def dtprint(s: str):
    iprint(s)


def iprint(obj):
    iprint_debug(obj)


def iprint_trace(obj):
    _iprint(obj, level=LLevel.Trace)


def iprint_debug(obj):
    _iprint(obj, level=LLevel.Debug)


def iprint_info(obj):

    _iprint(obj, level=LLevel.Info)


def iprint_warn(obj):
    _iprint(obj, level=LLevel.Warn)


def iprint_error(obj):
    _iprint(obj, level=LLevel.Error)


_log_inited = False


def _iprint(obj, level: LLevel):

    if AT._innerLoggerFun_ is not None:
        AT._innerLoggerFun_(obj, level)
        return
    else:
        pass

    global _log_inited
    if not _log_inited:
        _log_inited = True

        # 只在没有配置的时候起作用 不会重复执行
        logging.basicConfig(level=logging.NOTSET,
                            datefmt=AT.STRFMT_ISO_SEC_A,
                            format=AT._LOG_FORMAT_106,
                            )

        '''
        
    formatter = logging.Formatter('%(asctime)s.%(msecs)03d-%(name)s-%(filename)s-[line:%(lineno)d]'
                                  '-%(levelname)s-[日志信息]: %(message)s',
                                  datefmt='%Y-%m-%d,%H:%M:%S')
  
        
        '''

    # 将Trace级别提升一下。不知为何 NotSET的设置没有生效 并不会打印。
    if level == LLevel.Trace:
        level = LLevel.Debug

    logging.log(level.value, obj)

    return

# _myCntInfos = {}


'''

https://docs.python.org/3/howto/logging.html



class NoteItem(val ctime: Long, val msg: String?, val cnt: Long, val useTime: Long?) {

  val ctimeStr get() = AT.sdf_logger().format(Date(ctime))
}

class MyCntInfo(val title: String) {
  var cnt: Long = 0
  var items: LinkedList<NoteItem> = LinkedList()
  var utime: Long = 0

  val utimeStr get() = AT.sdf_logger().format(Date(utime))

}

'''

'''


@dataclass
class NoteItem:
    ctime: int
    cnt: int
    msg: str = None   # 这个字段是没有缺省值的
    useTime: int = None

    def ctimeStr(self) -> str:

        return 0 * 0


class DkCntInfoUtils(object):

    @staticmethod
    def recordCnt(_key, _cnt=1, _msg: str = None, _useTime: int = None):
        pass

    pass



'''




import numpy as np
import logging

from typing import List, Dict, Tuple, Union

import random
import time

import math
import random
from collections import namedtuple, deque
import re
import sys


import os
import subprocess
from queue import Queue
from queue import Empty
from pathlib import Path
import itertools

import time
from datetime import datetime
from datetime import date

from dataclasses import dataclass

from enum import Enum, unique

import base64
import argparse
import random
import time
import os


# ----------------------------------------

class LLevel(Enum):
    Trace = 0
    Debug = 10
    Info = 20
    Warn = 30
    Error = 40


    



import dknovautils
from dknovautils.commons import *

import beepy


DkAppVer = '0.1.3'

_unknown_err = '_unknown_err4035'


class AT(object):

    _innerLoggerFun_ = None

    _AstErrorCnt_ = 0

    __logger = None

    @classmethod
    def logger(cls, loggerName='dkn'):
        if cls.__logger is None:
            logger = logging.getLogger(loggerName)
            cls.__logger = logger

        return cls.__logger

    @staticmethod
    def log_loggerFun(loggerName='dkn', initInnerLoggerFun=True, beepError=True, beepWarn=False):
        '''
        创建一个logger 作为系统的缺省logger
        '''

        _logger = logging.getLogger(loggerName)

        def mloggerFun(obj, llevel):
            assert isinstance(llevel, LLevel)
            if False:
                pass
            elif llevel == LLevel.Trace:
                # 也用debug级别。只是在prod模式下可以关闭。
                _logger.debug(obj)
            elif llevel == LLevel.Debug:
                _logger.debug(obj)
            elif llevel == LLevel.Info:
                _logger.info(obj)
            elif llevel == LLevel.Warn:
                _logger.warning(obj)
                if beepWarn:
                    AT.beep_error_buffered()
            elif llevel == LLevel.Error:
                _logger.error(obj)
                if beepError:
                    AT.beep_error_buffered()
            else:
                assert False, f"bad {llevel}"

            pass

        if initInnerLoggerFun:
            AT._innerLoggerFun_ = mloggerFun

        return mloggerFun

    _LOG_FORMAT_100 = "%(asctime)s - %(levelname)s - %(message)s"
    _DATE_FORMAT_100 = "%m/%d/%Y %H:%M:%S %p"
    # _DATE_FORMAT_iso_simple = "%m/%d/%Y %H:%M:%S %p"

    STRFMT_ISO_COMPACT_SIMPLE = "%Y%m%dT%H%M%S"
    STRFMT_ISO_COMPACT_ALL_A = "%Y%m%dT%H%M%SS%fZ%Z"
    STRFMT_ISO_ALL_A = "%Y-%m-%dT%H:%M:%S.%f%Z"
    STRFMT_ISO_SEC_A = "%Y-%m-%dT%H:%M:%S"

    STRFMT_LOGGER_MS_A = "%Y-%m-%d %H:%M:%S.%f"

    _default_time_format = '%Y-%m-%d %H:%M:%S'

    _LOG_FORMAT_106 = '%(asctime)s.%(msecs)03d %(name)s %(threadName)s [%(levelname)s] %(message)s'

    '''
    
    dtstr = datetime.now().strftime("%Y%m%dT%H%M%S")

        
    '''

    @classmethod
    def log_basicConfig(cls, filename='tmp_test.log', loggerName='dkn', level=logging.DEBUG, initInnerLoggerFun=True):
        '''
        这个 dkn logger的设置好像不太好。
        暂时不用这个

        '''

        # AT.unsupported()

        logging.basicConfig(filename=filename, level=level,
                            format=cls._LOG_FORMAT_106,
                            datefmt=AT.STRFMT_ISO_SEC_A,
                            # datefmt=cls._DATE_FORMAT_100,
                            force=True)

        logger = logging.getLogger(loggerName)
        logger.setLevel(level)

        # # create console handler and set level to debug
        # ch = logging.StreamHandler()
        # ch.setLevel(logging.DEBUG)

        # # create formatter
        # formatter = logging.Formatter(
        #     cls._LOG_FORMAT_106)

        # # add formatter to ch
        # ch.setFormatter(formatter)

        # # add ch to logger
        # logger.addHandler(ch)

        mloggerFun = AT.log_loggerFun(loggerName, initInnerLoggerFun)

        return mloggerFun

    @staticmethod
    def assertAllNotNone(*args):
        assert isinstance(args, tuple)
        AT.assert_(all(_ is not None for _ in args),
                   'err7540 some value is None')

    @staticmethod
    def deg_to_rad(d):
        return d/180.0*math.pi

    __beep_last_time = 0

    @staticmethod
    def beep_error_buffered(tone='ping'):
        '''
        在0.5秒内最多播放一次 类似防止按钮双击的效果

        '''
        t = 0.5
        now = AT.fepochSecs()
        if now < AT.__beep_last_time+t:
            return
        else:
            AT.__beep_last_time = now
            AT.beep(tone)

    @staticmethod
    def beep(tone='ping'):
        beepy.beep(sound=tone)

    @staticmethod
    def beep2():
        #!wget -q -T 1 http://localhost:333/hello
        print('\7')

    @staticmethod
    def vassert_(b: bool, s: str = None):
        AT.assert_(b, s)        

    @classmethod
    def never(cls, s=None):
        AT.assert_(False,  s if s is not None else 'should never come here')

    @staticmethod
    def checksys():
        assert sys.version_info >= (3, 11)

    @classmethod
    def fepochMillis(cls) -> int:
        millisec = int(AT.fepochSecs() * 1000)
        return millisec

    _last_epochsecs = 0.0

    @classmethod
    def fepochSecs(cls) -> float:
        '''
         time.monotonic()不是epoch值。fuck
        # return time.monotonic()

        '''
        r = time.time()
        if r <= cls._last_epochsecs:
            return cls._last_epochsecs
        else:
            cls._last_epochsecs = r
            return r

    '''
        # dd/mm/YY H:M:S
    dts = now.strftime("%d/%m/%Y %H:%M:%S")
    print("date and time =", dts)

    # 20200101T120000
    dts = datetime.now().strftime("%Y%m%dT%H%M%S")
    print("date and time =", dts)
    
    
    '''

    @classmethod
    def sdf_isocompact_format_datetime(cls, dt=None, *, precise: str = 's'):
        assert precise in ['d', 's', 'ms', 'a'], 'err5554 bad precise'

        if dt is not None:
            AT.unsupported()

        dt = AT._now_dt()
        dts = dt.strftime(AT.STRFMT_ISO_COMPACT_ALL_A)

        if precise == 'a':
            pass
        elif precise == 'ms':
            dts = dts[:(8+0+1+6+4)]
        elif precise == 's':
            dts = dts[:(8+0+1+6)]
        elif precise == 'd':
            dts = dts[:(8)]
        else:
            AT.never()

        return dts

    @staticmethod
    def sdf_logger():
        AT.unimplemented()
        pass

    @staticmethod
    def _now_dt():
        dt = datetime.now()
        return dt

    @classmethod
    def sdf_logger_format_datetime(cls, dt: int = None, *, precise: str = 's') -> str:
        '''
https://strftime.org/    
https://docs.python.org/3/library/datetime.html#strftime-and-strptime-format-codes
https://man7.org/linux/man-pages/man3/strftime.3.html

2023-12-01T08:30:00.123456

"yyyy-MM-ddTHH:mm:ss"

date -I

d date
s seconds
ms millis
a all

        '''
        assert precise in ['d', 's', 'ms', 'a'], 'err5554 bad precise'

        if dt is not None:
            dt = datetime.fromtimestamp(dt/1000, tz=None)
        else:
            dt = datetime.now()

        dts = dt.strftime(AT.STRFMT_ISO_ALL_A)

        if precise == 'a':
            pass
        elif precise == 'ms':
            dts = dts[:(10+1+8+4)]
        elif precise == 's':
            dts = dts[:(10+1+8)]
        elif precise == 'd':
            dts = dts[:(10)]
        else:
            AT.never()

        return dts

    @staticmethod
    def assert_(b: bool, s: str = None):
        # 改成完善的形式

        assert isinstance(b, bool)

        if b:
            return

        AT._AstErrorCnt_ += 1
        msg = _unknown_err if s is None else s

        dknovautils.commons.iprint_error(msg)

        raise DkAstException(msg)

    @staticmethod
    def mychdir(s):
        assert isinstance(s, str) and len(s) > 0
        iprint_debug(f'chdir: {s}')
        os.chdir(s)

    @staticmethod
    def astTrace(b: bool, s: str):
        '''在prod中完全不需要的'''
        AT.assert_(b, s)

    @staticmethod
    def unsupported(s: str = 'feature'):
        AT.assert_(False, f'err8255 unsupported [{s}]')

    @staticmethod
    def unimplemented(s: str = 'feature'):
        AT.assert_(False, f'err4173 unimplemented [{s}]')

    # @staticmethod
    # def f_matploglib_logger():
    #     logger = logging.getLogger('matplotlib.font_manager')
    #     logger.setLevel(level=logging.INFO)

    @staticmethod
    def log_conf_matploglib_logger(level=logging.INFO):
        logger = logging.getLogger('matplotlib.font_manager')
        logger.setLevel(level=level)        

    VERSION = DkAppVer


class DkAstException(Exception):
    pass


'''

LOG_FORMAT = "%(asctime)s - %(levelname)s - %(message)s"
DATE_FORMAT = "%m/%d/%Y %H:%M:%S %p"

logging.basicConfig(filename='/mnt/d/tmp/demo0726.log', level=logging.DEBUG, format=LOG_FORMAT, datefmt=DATE_FORMAT)





'''



from dknovautils.dk_imports import *

# 导入module避免某些循环import的问题
from dknovautils import commons


from IPython.core.getipython import get_ipython


class DkIpyUtils(object):

    @staticmethod
    def hello(a):
        commons.iprint_info("hello")
        return a  # 为了测试该文件是否正常

    @staticmethod
    def mfc(cmd: str, logcmd: bool = True):
        DkIpyUtils.mfr(cmd, logcmd)

    _cmd_id = 1000

    @staticmethod
    def mfr(cmd, logcmd=True) -> any:
        assert isinstance(cmd, str) and len(cmd) > 0, 'err3581'
        DkIpyUtils._cmd_id += 1
        if logcmd:
            commons.iprint_info(f'run cmd begin {DkIpyUtils._cmd_id}: {cmd}')

        r = get_ipython().getoutput(cmd)

        if logcmd:
            commons.iprint_info(f'run cmd end {DkIpyUtils._cmd_id}')

        return r


def dk_mfc(cmd, logcmd): DkIpyUtils.mfc(cmd, logcmd)


def dk_mfr(cmd, logcmd): return DkIpyUtils.mfr(cmd, logcmd)

'''

!{cmd}

r=get_ipython().getoutput('{cmd}')



不能用 %cd 格式化会出错 用 os.chdir()

不用 echo 用 iprint






'''



#!/usr/bin/env python

import re
import setuptools

from distutils.core import setup
from setuptools import find_packages

DkAppVer = '0.1.3'

with open("README.md", "r", encoding="utf-8") as f:
    long_description = f.read()

setuptools.setup(
    name="dknovautils",
    version=DkAppVer,
    author="dknova",
    author_email="dknova@example.com",
    description="This is the tools for dknova.",
    long_description=long_description,
    # long_description_content_type="text/markdown",
    url="http://example.com",
    install_requires=[
        # 'requests!=2.9.0',
        # 'lxml',
        # 'monotonic>=1.5',
        'numpy',
        'beepy',
    ],
    packages=setuptools.find_packages(exclude=("test")),
    classifiers=(
        "License :: OSI Approved :: MIT License",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",

        "Programming Language :: Python :: 3.7"
    ),
    exclude_package_data={'': ["*.txt", "*.adoc", "*.sh", "dknovautils/test.py",
                               "dknovautils/config.txt", "dknovautils/*.sh", "dknovautils/test*.py"]},
)
